       /*
        * sbox.c: port I/O, switch switchbox on or off
	* 
        * (compile:  gcc -O2 -o sbox sbox.c)
        */

       #include <stdio.h>
       #include <unistd.h>
       #include <asm/io.h>

       #define ser_par_off 0;
       #define ser_on 2;
       #define par_on 8;

       int main(argc, argv)
           int argc;
	   char *argv[];

       {
	    int BASEPORT = 0;
	    int on_off = 0;
	    int ser_par = 0;
	    int data = 0;
	    int verbose = 0;
	    int before = 0;
	    int after = 0;
	    int state;
	    char *version = "sbox, version 1.2, October 21st, 2002 (chris@gensler.to)\n";
	    char *usage = "usage: sbox [options] com{1234}|lpt{123} [on|off|1|0]\n" \
			  "hint: if you omit [on|off], the actual state will be printed.\n" \
			  "options:\n" \
			  "       -v            be verbose - print 2-line, tab-separated result\n" \
			  "                     see verbose output itself for explanations!\n" \
			  "       --help        show this help and exit\n" \
			  "       --version     print version and exit\n";
	      
            argv++, argc--;
	    while (argc > 0) {
		if (strcmp(*argv, "--help") == 0) {
		    printf(usage);
		    exit(0);
		}
		if (strcmp(*argv, "--version") == 0) {
		    printf(version);
		    exit(0);
		}
		if (strcmp(*argv, "-v") == 0) {
		    verbose = 1;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "on") == 0 || strcmp(*argv, "ON") == 0 || strcmp(*argv, "1") == 0) {
		    on_off = 1;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "off") == 0 || strcmp(*argv, "OFF") == 0 || strcmp(*argv, "0") == 0) {
		    on_off = 2;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "com1") == 0 || strcmp(*argv, "COM1") == 0 ) {
		    ser_par = 1;
		    BASEPORT = 0x3f8 + 4;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "com2") == 0 || strcmp(*argv, "COM2") == 0 ) {
		    ser_par = 1;
		    BASEPORT = 0x2f8 + 4;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "com3") == 0 || strcmp(*argv, "COM3") == 0 ) {
		    ser_par = 1;
		    BASEPORT = 0x3e8 + 4;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "com4") == 0 || strcmp(*argv, "COM4") == 0 ) {
		    ser_par = 1;
		    BASEPORT = 0x2e8 + 4;
		    argv++, argc--;
		    continue;
		}	
		if (strcmp(*argv, "lpt1") == 0 || strcmp(*argv, "LPT1") == 0 ) {
		    ser_par = 2;
		    BASEPORT = 0x378;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "lpt2") == 0 || strcmp(*argv, "LPT2") == 0 ) {
		    ser_par = 2;
		    BASEPORT = 0x278;
		    argv++, argc--;
		    continue;
		}
		if (strcmp(*argv, "lpt3") == 0 || strcmp(*argv, "LPT3") == 0 ) {
		    ser_par = 2;
		    BASEPORT = 0x3bc;
		    argv++, argc--;
		    continue;
		}
		fprintf(stderr,usage);
		exit(1);
	    }
	
	    switch (on_off) {
		case 1: {
        	    switch (ser_par) {
			case 1: {data=ser_on; break;}
		        case 2: {data=par_on; break;}
		    }
		    break;
		}
		case 2: {
		    data=ser_par_off;
		    break;
		}
	    }

	    if (BASEPORT == 0 || ser_par == 0) {fprintf(stderr,usage);exit(1);}

	    /* Get access to the ports, need root rights */
    	    if (ioperm(BASEPORT, 1, 1)) {perror("(root?) ioperm"); exit(1);}

    	    /* Read from the status port (BASE+1) before setting */
	    before=inb(BASEPORT);

	    /* if there's a mode given to be set ... */
	    if (on_off > 0) {
	    
    		/* set the data - this switches the box! */
    	        outb(data, BASEPORT);

    	        /* Sleep for a while (100 ms) */
    	        usleep(100000);

	    }

    	    /* Read from the status port (BASE+1) after setting - just to be displayed for check! */
	    after=inb(BASEPORT);
	    
	    /* special handling of mode request */
	    /* prints actual mode (on|off) when no mode is given to set */
	    if (on_off == 0) {
		state=0;
        	switch (ser_par) {
		    case 1: {state=after & ser_on; break; data=ser_on;}
		    case 2: {state=after & par_on; break; data=par_on;}
		}
		// print short info test on verbose output
		if (verbose == 1) {printf("state of port is ");};
		if (state > 0) {printf("on\n"); on_off=1;} else {printf("off\n"); on_off=2;};
		if (verbose == 1) {printf("hint: no mode given, so no data is set:\n");};
	    }
	    
	    /* verbose output: print informational data separated by tabs */
	    /* those lines both start with an "#" (no other will!) */
	    /* to easily get them grep'd, followed by "1" or "2" for head and data */
	    if (verbose == 1) {
		printf("legend: port is hex, old, set and new are decimal data\n");
		printf("        ser_par and on_off are 1 (ser/on) or 2 (par/off)\n");
		printf("        head is prefixed '#1', data is prefixed '#2'\n");
		printf("        (use \"grep '^#'\" or \"grep '^#2'\" to extract)\n");
		printf("#1\tport\tser_par\ton_off\told\tset\tnew\n");
		printf("#2\t0x%x\t%d\t%d\t%d\t%d\t%d\n",BASEPORT,ser_par,on_off,before,data,after);
	    };

    	    /* We don't need the ports anymore */
    	    if (ioperm(BASEPORT, 1, 0)) {perror("ioperm"); exit(1);}

    	    exit(0);
	}

	/* end of example.c */





